/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "DocumentHolder.h"

DocumentHolder::DocumentHolder(const Document::KeywordList& keywords, C4sEngineFunctionList* engine_list, C4sPreferences* preferences):
  m_document(keywords, preferences, engine_list),
  m_function_bar(c4s_function_bar_new(engine_list, m_document.get_function_list(), m_document.get_param_info()))
{
  g_signal_connect_after(
    G_OBJECT(m_function_bar),
    "custom-function-selected",
    G_CALLBACK(on_function_selected_static),
    this
  );

  gtk_box_pack_start(GTK_BOX(gobj()), GTK_WIDGET(m_function_bar), FALSE, FALSE, 0);
  pack_start(m_document, Gtk::PACK_EXPAND_WIDGET);
  set_spacing(4);
}

DocumentHolder::DocumentHolder(const std::string& file, const Document::KeywordList& keywords, C4sEngineFunctionList* engine_list, C4sPreferences* preferences):
  m_document(file, keywords, preferences, engine_list),
  m_function_bar(c4s_function_bar_new(engine_list, m_document.get_function_list(), m_document.get_param_info()))
{
  g_signal_connect_after(
    G_OBJECT(m_function_bar),
    "custom-function-selected",
    G_CALLBACK(on_function_selected_static),
    this
  );

  gtk_box_pack_start(GTK_BOX(gobj()), GTK_WIDGET(m_function_bar), FALSE, FALSE, 0);
  pack_start(m_document, Gtk::PACK_EXPAND_WIDGET);
  set_spacing(4);
}

void DocumentHolder::on_function_selected(const C4sCustomFunction* function)
{
  m_document.jump_to_function(function);
}
