/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-iconmanager.h>

#include "MainMenu.h"

namespace
{
  Glib::ustring ui_desc =
    "<ui>"
    " <menubar name=\"MainMenu\">"
    "  <menu action=\"MenuFile\">"
    "   <menuitem action=\"FileNew\" />"
    "   <menuitem action=\"FileOpen\" />"
    "   <menuitem action=\"FileSave\" />"
    "   <menuitem action=\"FileSaveAs\" />"
    "   <menuitem action=\"FileSaveAll\" />"
    "   <menuitem action=\"FileClose\" />"
    "   <separator />"
    "   <menuitem action=\"Quit\" />"
    "  </menu>"
    "  <menu action=\"MenuEdit\">"
    "   <menuitem action=\"EditUndo\" />"
    "   <menuitem action=\"EditRedo\" />"
    "   <separator />"
    "   <menuitem action=\"EditCut\" />"
    "   <menuitem action=\"EditCopy\" />"
    "   <menuitem action=\"EditPaste\" />"
    "   <separator />"
    "   <menuitem action=\"EditFind\" />"
    "   <menuitem action=\"EditReplace\" />"
    "   <menuitem action=\"EditGoto\" />"
    "   <separator />"
    "   <menuitem action=\"EditPreferences\" />"
    "  </menu>"
    "  <menu action=\"MenuView\">"
    "   <menuitem action=\"ViewParamToolTip\" />"
    "   <menuitem action=\"ViewStatusBar\" />"
    "  </menu>"
    "  <menu action=\"MenuHelp\">"
    "   <menuitem action=\"HelpAbout\" />"
    /* Accelerators only work with menu items... I rather would like to
     * hide this menu item and make this option only available via
     * the toolbar or the accelerator... */
    "   <separator />"
    "   <menuitem action=\"HelpDocumentation\" />"
    "  </menu>"
    " </menubar>"
    " <toolbar name=\"MainTool\">"
    "  <toolitem action=\"FileNew\" />"
    "  <toolitem action=\"FileOpen\" />"
    "  <toolitem action=\"FileSave\" />"
    "  <toolitem action=\"FileSaveAll\" />"
    "  <separator />"
    "  <toolitem action=\"EditUndo\" />"
    "  <toolitem action=\"EditRedo\" />"
    "  <separator />"
    "  <toolitem action=\"EditCopy\" />"
    "  <toolitem action=\"EditCut\" />"
    "  <toolitem action=\"EditPaste\" />"
    "  <separator />"
    "  <toolitem action=\"EditFind\" />"
    "  <toolitem action=\"EditReplace\" />"
    "  <separator />"
    "  <toolitem action=\"HelpDocumentation\" />"
    " </toolbar>"
    "</ui>";
}

MainMenu::MainMenu():
  m_ui_manager(Gtk::UIManager::create() ),
  m_group_file(Gtk::ActionGroup::create() ),
  m_group_edit(Gtk::ActionGroup::create() ),
  m_group_view(Gtk::ActionGroup::create() ),
  m_group_help(Gtk::ActionGroup::create() ),
  // FileActions
  action_file(Gtk::Action::create("MenuFile", "_File")),
  action_file_new(Gtk::Action::create("FileNew", Gtk::Stock::NEW, "New", "Creates an empty script")),
  action_file_open(Gtk::Action::create("FileOpen", Gtk::Stock::OPEN, "Open...", "Opens an existing script")),
  action_file_save(Gtk::Action::create("FileSave", Gtk::Stock::SAVE, "Save", "Saves the current script")),
  action_file_save_as(Gtk::Action::create("FileSaveAs", Gtk::Stock::SAVE_AS, "Save as...", "Saves the current script to another location")),
  action_file_save_all(Gtk::Action::create("FileSaveAll", Gtk::Stock::SAVE, "Save All", "Saves all open files")),
  action_file_close(Gtk::Action::create("FileClose", Gtk::Stock::CLOSE, "Close", "Close the current document")),
  action_quit(Gtk::Action::create("Quit", Gtk::Stock::QUIT, "Quit")),
  // EditActions
  action_edit(Gtk::Action::create("MenuEdit", "_Edit")),
  action_edit_undo(Gtk::Action::create("EditUndo", Gtk::Stock::UNDO, "Undo", "Undoes the last action")),
  action_edit_redo(Gtk::Action::create("EditRedo", Gtk::Stock::REDO, "Redo", "Redoes the last undone action")),
  action_edit_cut(Gtk::Action::create("EditCut", Gtk::Stock::CUT, "Cut", "Cuts the selected text")),
  action_edit_copy(Gtk::Action::create("EditCopy", Gtk::Stock::COPY, "Copy", "Copies the selection into the clipboard")),
  action_edit_paste(Gtk::Action::create("EditPaste", Gtk::Stock::PASTE, "Paste", "Inserts the text from the clipboard")),
  action_edit_find(Gtk::Action::create("EditFind", Gtk::Stock::FIND, "Find...", "Searches the text for a given string")),
  action_edit_replace(Gtk::Action::create("EditReplace", Gtk::Stock::FIND_AND_REPLACE, "Find and replace...", "Find text and replace it with other text")),
  action_edit_goto(Gtk::Action::create("EditGoto", Gtk::Stock::JUMP_TO, "Goto...", "Jumps to a specified line or function")),
  action_edit_preferences(Gtk::Action::create("EditPreferences", Gtk::Stock::PREFERENCES, "Preferences...", "Opens a dialog to customize C4Scripter")),
  // ViewActions
  action_view(Gtk::Action::create("MenuView", "_View")),
  action_view_param_tool_tip(Gtk::ToggleAction::create("ViewParamToolTip", Gtk::Stock::DIALOG_INFO, "Tooltips", "Switches the parameter tooltips on/off")),
  action_view_statusbar(Gtk::ToggleAction::create("ViewStatusBar", Gtk::Stock::INFO, "Statusbar", "Switches the statusbar on/off")),
  // HelpActions
  action_help(Gtk::Action::create("MenuHelp", "_Help")),
  action_help_about(Gtk::Action::create("HelpAbout",Gtk::Stock::ABOUT, "About...")),
  action_help_documentation(Gtk::Action::create("HelpDocumentation", Gtk::StockID(C4S_STOCK_DOCUMENTATION), "Documentation", "View documentation to the currently selected function"))
{
  //File...
  m_group_file->add(action_file);
  m_group_file->add(action_file_new);
  m_group_file->add(action_file_open);
  m_group_file->add(action_file_save);
  m_group_file->add(action_file_save_as);
  m_group_file->add(action_file_save_all, Gtk::AccelKey("<control><shift>S", "<Actions>/MenuFile/FileSaveAll"));
  m_group_file->add(action_file_close);
  m_group_file->add(action_quit);
  //Edit...
  m_group_edit->add(action_edit);
  m_group_edit->add(action_edit_undo, Gtk::AccelKey("<control>Z", "<Actions>/MenuEdit/EditUndo"));
  m_group_edit->add(action_edit_redo, Gtk::AccelKey("<control><shift>Z", "<Actions>/MenuEdit/EditRedo"));
  m_group_edit->add(action_edit_cut);
  m_group_edit->add(action_edit_copy);
  m_group_edit->add(action_edit_paste);
  m_group_edit->add(action_edit_find);
  m_group_edit->add(action_edit_replace);
  m_group_edit->add(action_edit_goto, Gtk::AccelKey("<control>I", "<Actions>/MenuEdit/EditGoto"));
  m_group_edit->add(action_edit_preferences);
  //View...
  m_group_view->add(action_view);
  m_group_view->add(action_view_param_tool_tip);
  m_group_view->add(action_view_statusbar);
  //Help...
  m_group_help->add(action_help);
  m_group_help->add(action_help_about);
  m_group_help->add(action_help_documentation, Gtk::AccelKey("F1", "<Actions>/HelpDocumentation"));

  m_ui_manager->add_ui_from_string(ui_desc); // Laden das Ding von oben in den UIManager
  m_ui_manager->insert_action_group(m_group_file);
  m_ui_manager->insert_action_group(m_group_edit);
  m_ui_manager->insert_action_group(m_group_view);
  m_ui_manager->insert_action_group(m_group_help);

	// Das sind dann die beiden widgets die wir irgendwohin packen koennen, oben würd ich mal sagen? ;) Ja. Aber MainMenu ist erstmal nur eine VBox. Das heisst wir packen die beiden bars in die VBox und MainWindow haut uns als VBox nach oben ins Window. k
  m_menubar = static_cast<Gtk::MenuBar*>(m_ui_manager->get_widget("/MainMenu") );
  m_toolbar = static_cast<Gtk::Toolbar*>(m_ui_manager->get_widget("/MainTool") );

/*  action_file_new->signal_activate().connect(sigc::mem_fun(m_handler, &FileHandler::new_file) );
  action_file_open->signal_activate().connect(sigc::mem_fun(m_handler, &FileHandler::open_file) );
  action_file_save->signal_activate().connect(sigc::mem_fun(m_handler, &FileHandler::save_file) );
  action_file_save_as->signal_activate().connect(sigc::mem_fun(m_handler, &FileHandler::save_file_as) );
  action_file_save_all->signal_activate().connect(sigc::mem_fun(m_handler, &FileHandler::save_all) );*/

  pack_start(*m_menubar, Gtk::PACK_SHRINK);
  pack_start(*m_toolbar, Gtk::PACK_SHRINK);
}

Glib::RefPtr<Gtk::AccelGroup> MainMenu::get_accel_group()
{
	return m_ui_manager->get_accel_group();
}
