/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MAINWINDOW_H_INCLUDED
#define MAINWINDOW_H_INCLUDED

#include <gtkmm.h>

#include <c4scripter/c4s-iconmanager.h>
#include <c4scripter/c4s-remotecomm.h>
#include <c4scripter/c4s-preferences.h>

#include "MainMenu.h"
#include "FileHandler.h"
#include "FileHolder.h"
#include "DocumentControl.h"
#include "ViewControl.h"
#include "ParamTip.h"
#include "Statusbar.h"

class MainWindow: public Gtk::Window
{
public:
  MainWindow(int argc, char* argv[], C4sRemoteComm* comm);
  ~MainWindow();

  FileHandler& get_file_handler() { return m_file_handler; }

private:
  virtual bool on_delete_event(GdkEventAny* event);

  void on_quit();
  void on_about();
  void on_documentation();

  static void on_file_received_static(C4sRemoteComm* comm, const gchar* file, gpointer user_data) { static_cast<MainWindow*>(user_data)->on_file_received(file); }
  void on_file_received(const std::string& file);

  void on_document_changed(DocumentHolder* holder);
  void on_modified_changed();
  void on_title_changed();
  void update_title();

  Gtk::VBox m_top_box;
  GdkPixbuf* m_logo;
  C4sIconManager* m_icon_manager;
  C4sPreferences* m_preferences;
  MainMenu m_main_menu;
  FileHolder m_file_holder;
  FileHandler m_file_handler;
  DocumentControl m_doc_control;
  ViewControl m_view_control;
  Statusbar m_statusbar;
  ParamTip m_paramtip;

  sigc::connection m_title_conn;
  sigc::connection m_modified_conn;
};

#endif // MAINWINDOW_H_INCLUDED
