/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef STATUSBAR_H_INCLUDED
#define STATUSBAR_H_INCLUDED

#include <gtkmm.h>
#include <c4scripter/c4s-preferences.h>

#include "FileHolder.h"

class Statusbar: public Gtk::Statusbar
{
public:
  Statusbar(FileHolder& file_holder, C4sPreferences* preferences);

private:
  static void on_notify(GObject* object, GParamSpec* pspec, gpointer data);

  void on_statusbar_visibility();
  void on_document_changed(DocumentHolder* document);
  void on_cursor_moved(const Gtk::TextIter& iter, const Glib::RefPtr<Gtk::TextMark>& mark);
  void on_text_inserted(const Gtk::TextIter& iter, const Glib::ustring& text, int bytes);
  void on_text_erased(const Gtk::TextIter& begin, const Gtk::TextIter& end);

  void update_statusbar(const Gtk::TextIter* pos);

  Gtk::Statusbar m_cursor_position_bar;

  FileHolder& m_file_holder;
  C4sPreferences* m_preferences;
  Document* m_current_document;

  sigc::connection m_conn_cursor_moved;
  sigc::connection m_conn_text_inserted;
  sigc::connection m_conn_text_erased;
};

#endif // STATUSBAR_H_INCLUDED
