/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ViewControl.h"

ViewControl::ViewControl(MainMenu& menu, C4sPreferences* preferences, Gtk::Window& parent):
  m_menu(menu), m_preferences(preferences), m_parent(parent), m_changing(false)
{
  menu.action_view_param_tool_tip->set_active(c4s_preferences_get_view_tooltips(m_preferences));
  menu.action_view_statusbar->set_active(c4s_preferences_get_view_statusbar(m_preferences));

  menu.action_view_param_tool_tip->signal_activate().connect(sigc::mem_fun(*this, &ViewControl::OnParamTip));
  menu.action_view_statusbar->signal_activate().connect(sigc::mem_fun(*this, &ViewControl::OnStatusBar));

  // External change
  g_signal_connect(
    G_OBJECT(m_preferences),
    "notify",
    G_CALLBACK(on_notify),
    this
  );
}

void ViewControl::on_notify(GObject* object, GParamSpec* pspec, gpointer data)
{
  if(strcmp(pspec->name, "view-tooltips"))
    static_cast<ViewControl*>(data)->OnParamTipChanged();
  else if(strcmp(pspec->name, "view-statusbar"))
    static_cast<ViewControl*>(data)->OnStatusBarChanged();
}

void ViewControl::OnParamTip()
{
  if(m_changing) return;
  m_changing = true;
  c4s_preferences_set_view_tooltips(
    m_preferences,
    !c4s_preferences_get_view_tooltips(m_preferences)
  );
  m_changing = false;
}

void ViewControl::OnStatusBar()
{
  if(m_changing) return;

  m_changing = true;
  c4s_preferences_set_view_statusbar(
    m_preferences,
    !c4s_preferences_get_view_statusbar(m_preferences)
  );
  m_changing = false;
}

void ViewControl::OnParamTipChanged()
{
  if(m_changing) return;

  m_changing = true;
  m_menu.action_view_param_tool_tip->set_active(c4s_preferences_get_view_tooltips(m_preferences));
  m_changing = false;
}

void ViewControl::OnStatusBarChanged()
{
  if(m_changing) return;

  m_changing = true;
  m_menu.action_view_statusbar->set_active(c4s_preferences_get_view_statusbar(m_preferences));
  m_changing = false;
}
