/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-textutil.h>

#include <gtk/gtkmain.h>
#include <glib/gunicode.h>

#include <errno.h>
#include <string.h>

gboolean
c4s_file_write8(FILE* stream,
                guint8 value,
                GError** error)
{
  size_t result;
  result = fwrite(&value, sizeof(guint8), 1, stream);

  if(result == EOF)
  {
    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < 1)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Failed to write to file"
    );

    return FALSE;
  }

  return TRUE;
}

gboolean
c4s_file_write16(FILE* stream,
                 guint16 value,
                 GError** error)
{
  size_t result;
  result = fwrite(&value, sizeof(guint16), 1, stream);

  if(result == EOF)
  {
    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < 1)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Failed to write to file"
    );

    return FALSE;
  }

  return TRUE;
}

gboolean
c4s_file_write32(FILE* stream,
                 guint32 value,
                 GError** error)
{
  size_t result;
  result = fwrite(&value, sizeof(guint32), 1, stream);

  if(result == EOF)
  {
    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < 1)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Failed to write to file"
    );

    return FALSE;
  }

  return TRUE;
}

gboolean
c4s_file_write_string(FILE* stream,
                      const gchar* value,
                      GError** error)
{
  size_t result;
  size_t len;

  len = strlen(value);
  if(len >= 0xff)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      1,
      "String '%s' is too long",
      value
    );

    return FALSE;
  }

  if(!c4s_file_write8(stream, (guint8)len, error)) return FALSE;

  result = fwrite(value, sizeof(gchar), len, stream);

  if(result == EOF)
  {
    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < len)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Failed to write to file"
    );

    return FALSE;
  }

  return TRUE;
}

gboolean
c4s_file_read8(FILE* stream,
               guint8* value,
               GError** error)
{
  size_t result;
  result = fread(value, sizeof(guint8), 1, stream);

  if(result == EOF)
  {
    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < 1)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Unexpected end of file"
    );

    return FALSE;
  }

  return TRUE;
}

gboolean
c4s_file_read16(FILE* stream,
                guint16* value,
                GError** error)
{
  size_t result;
  result = fread(value, sizeof(guint16), 1, stream);

  if(result == EOF)
  {
    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < 1)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Unexpected end of file"
    );

    return FALSE;
  }

  return TRUE;
}

gboolean
c4s_file_read32(FILE* stream,
                guint32* value,
                GError** error)
{
  size_t result;
  result = fread(value, sizeof(guint32), 1, stream);

  if(result == EOF)
  {
    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < 1)
  {
    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Unexpected end of file"
    );

    return FALSE;
  }

  return TRUE;
}

gboolean
c4s_file_read_string(FILE* stream,
                     gchar** value,
                     GError** error)
{
  size_t result;
  guint8 len;

  if(!c4s_file_read8(stream, &len, error)) return FALSE;

  *value = g_malloc( (len + 1) * sizeof(gchar));
  result = fread(*value, sizeof(gchar), len, stream);

  if(result == EOF)
  {
    g_free(*value);

    g_set_error(
      error,
      G_FILE_ERROR,
      g_file_error_from_errno(errno),
      "%s",
      g_strerror(errno)
    );

    return FALSE;
  }

  if(result < len)
  {
    g_free(*value);

    g_set_error(
      error,
      g_quark_from_static_string("C4S_FILE_ERROR"),
      0,
      "Unexpected end of file"
    );

    return FALSE;
  }

  (*value)[len] = '\0';
  return TRUE;
}
