/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-valuetype.h>
#include <c4scripter/c4s-textutil.h>

#include <ctype.h>

GType
c4s_value_type_get_type(void)
{
  static GType etype = 0;
  if(etype == 0)
  {
    static const GFlagsValue values[] = {
      { C4S_VALUE_TYPE_UNKNOWN, "C4S_VALUE_TYPE_UNKNOWN", "unknown" },
      { C4S_VALUE_TYPE_INT, "C4S_VALUE_TYPE_INT", "int" },
      { C4S_VALUE_TYPE_ID, "C4S_VALUE_TYPE_ID", "id" },
      { C4S_VALUE_TYPE_BOOL, "C4S_VALUE_TYPE_BOOL", "bool" },
      { C4S_VALUE_TYPE_STRING, "C4S_VALUE_TYPE_STRING", "string" },
      { C4S_VALUE_TYPE_OBJECT, "C4S_VALUE_TYPE_OBJECT", "object" },
      { C4S_VALUE_TYPE_ARRAY, "C4S_VALUE_TYPE_ARRAY", "array" },
      { C4S_VALUE_TYPE_ANY, "C4S_VALUE_TYPE_ANY", "any" },
      { C4S_VALUE_TYPE_REF, "C4S_VALUE_TYPE_REF", "&" },
      { 0, NULL, NULL }
    };

    etype = g_flags_register_static("C4sValueType", values);
  }

  return etype;
}

const gchar*
c4s_value_type_to_string(C4sValueType type)
{
  switch(type)
  {
  case C4S_VALUE_TYPE_INT: return "int";
  case C4S_VALUE_TYPE_ID: return "id";
  case C4S_VALUE_TYPE_BOOL: return "bool";
  case C4S_VALUE_TYPE_STRING: return "string";
  case C4S_VALUE_TYPE_OBJECT: return "object";
  case C4S_VALUE_TYPE_ARRAY: return "array";
  case C4S_VALUE_TYPE_ANY: return "any";
  case C4S_VALUE_TYPE_INT | C4S_VALUE_TYPE_REF: return "int&";
  case C4S_VALUE_TYPE_ID | C4S_VALUE_TYPE_REF: return "id&";
  case C4S_VALUE_TYPE_BOOL | C4S_VALUE_TYPE_REF: return "bool&";
  case C4S_VALUE_TYPE_STRING | C4S_VALUE_TYPE_REF: return "string&";
  case C4S_VALUE_TYPE_OBJECT | C4S_VALUE_TYPE_REF: return "object&";
  case C4S_VALUE_TYPE_ARRAY | C4S_VALUE_TYPE_REF: return "array&";
  case C4S_VALUE_TYPE_ANY | C4S_VALUE_TYPE_REF: return "any&";
  default: return "unknown"; /*g_assert_not_reached();*/
  }
}

C4sValueType
c4s_value_type_from_string(const gchar* name,
                           gsize len)
{
  gsize end;
  C4sValueType type;

  end = 0;
  while(end < len && isalpha(name[end]))
    ++ end;

  if(c4s_substr_cmp(name, end, "int", 3))
    type = C4S_VALUE_TYPE_INT;
  else if(c4s_substr_cmp(name, end, "id", 2))
    type = C4S_VALUE_TYPE_ID;
  else if(c4s_substr_cmp(name, end, "bool", 4))
    type = C4S_VALUE_TYPE_BOOL;
  else if(c4s_substr_cmp(name, end, "string", 6))
    type = C4S_VALUE_TYPE_STRING;
  else if(c4s_substr_cmp(name, end, "object", 6))
    type = C4S_VALUE_TYPE_OBJECT;
  else if(c4s_substr_cmp(name, end, "array", 5))
    type = C4S_VALUE_TYPE_ARRAY;
  else if(c4s_substr_cmp(name, end, "any", 3))
    type = C4S_VALUE_TYPE_ANY;
  else
    return C4S_VALUE_TYPE_UNKNOWN;

  while(end < len && isspace(name[end])) ++ end;
  if(end < len && name[end] == '&') type |= C4S_VALUE_TYPE_REF;

  return type;
}
