/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-remotecomm.h>
#include <c4scripter/comm/c4s-comm.h>

#include "MainWindow.h"

#include <gtkmm.h>
#include <iostream>

#include <signal.h>

MainWindow* _global_wnd;

void segv_handler(int signal)
{
  if(_global_wnd == NULL) return;

  Gtk::MessageDialog dialog(
    *_global_wnd,
    "The application crashed!",
    false,
    Gtk::MESSAGE_ERROR,
    Gtk::BUTTONS_OK,
    true
  );

  dialog.set_title("C4Scripter crashed");

  dialog.set_secondary_text(
    "Please send a bug report to Bernhard Bonigl <boni@bbgen.net> and/or "
    "Armin Burgmeier <armin@arbur.net> including an explanation what you "
    "have done before C4Scripter crashed and the files you had open.\n\n"
    "You will get the possibility to save your modified files after having "
    "clicked OK."
  );

  dialog.run();

  _global_wnd->get_file_handler().close_all_documents();
  std::exit(EXIT_FAILURE);
}

int main(int argc, char* argv[])
{
  Gtk::Main kit(argc, argv);

  C4sRemoteComm* comm;
#ifdef G_OS_WIN32
  comm = c4s_remote_comm_new(&C4S_COMM_BACKEND_WIN32);
#else
  comm = c4s_remote_comm_new(&C4S_COMM_BACKEND_UNIX);
#endif

  if(c4s_remote_comm_connect(comm) == TRUE)
  {
    c4s_remote_comm_send(comm, argc - 1, argv + 1);
  }
  else
  {
    std::auto_ptr<MainWindow> window;
    signal(SIGSEGV, segv_handler);

    window.reset(new MainWindow(argc - 1, argv + 1, comm));
    _global_wnd = window.get();

    c4s_remote_comm_open(comm, window->gobj());
    kit.run(*window);

    _global_wnd = NULL;
  }

  g_object_unref(G_OBJECT(comm));
  return 0;
}
